console.log('background.js loaded');

var windows = [];
var timeoutId = -1;

function getTrackedWindow(windowId) {
	for (var i = 0; i < windows.length; i++) {
		if (windows[i].id == windowId)
			return windows[i];
	}
	return undefined;
}

function getTrackedWindowByPath(server, module, bucket) {
	for (var i = 0; i < windows.length; i++) {
		if (windows[i]._server == server && windows[i]._module == module && windows[i]._bucket == bucket)
			return windows[i];
	}
	return undefined;
}

function untrackWindow(windowId) {
	for (var i = 0; i < windows.length; i++) {
		if (windows[i].id == windowId) {
			windows.splice(i, 1);
			return true;
		}
	}
	return false;
}

function setDelayedSave(win) {
	clearTimeout(win.saveTimerId);

	win.saveTimerId = setTimeout(function() {
		save(win);
	}, 2000);
}

function save(win) {
	chrome.tabs.query({windowId:win.id}, function(tabs) {
		tabs.sort(function(a, b) {
			return a.index - b.index;
		});

		links = [];
		for (var i = 0; i < tabs.length; i++) {
			links.push(tabs[i].url);
		}

		var data = btoa(JSON.stringify(links));

		var xhr = new XMLHttpRequest();
		xhr.open('GET', 'https://'+win._server+'/data?do=bucketsavesess&module='+win._module+'&bucket='+win._bucket+'&links='+data, true);
		xhr.onreadystatechange = function() {
			//console.log(xhr);
			if (xhr.readyState == 4) {
				console.log('saved', xhr.responseText);
				for (var i = 0; i < tabs.length; i++) {
					chrome.pageAction.setIcon({ tabId:tabs[i].id, path:'node-19-fade.png' });
				}
			}
		}
		xhr.send();
	});
}

chrome.runtime.onMessage.addListener(function(request, sender, sendResponse) {
	console.log(request, sender);

	if (request.do == 'launchSess') {
		var win = getTrackedWindowByPath(request.server, request.module, request.bucket);
		if (win) {
			chrome.windows.update(win.id, {'focused':true});
		} else {
			chrome.windows.getCurrent(function(sourceWin) {
				chrome.windows.create({'url':request.links, 'incognito':sourceWin.incognito}, function(win) {
					console.log('created:', win);
					win._server = request.server;
					win._module = request.module;
					win._bucket = request.bucket;
					windows.push(win);
				});
			});
		}
	}

	sendResponse(request);
});

chrome.tabs.onUpdated.addListener(function(tabId, changeInfo, tab) {
	var win = getTrackedWindow(tab.windowId);
	if (win) {
		console.log('tabs.onUpdated', changeInfo, tab);
		chrome.pageAction.setIcon({ tabId:tab.id, path:'node-19.png' });
		chrome.pageAction.show(tab.id);
		setDelayedSave(win);
	}
});

chrome.tabs.onRemoved.addListener(function(tabId, removeInfo) {
	var win = getTrackedWindow(removeInfo.windowId);
	if (win) {
		console.log('tabs.onRemoved', tabId, win, 'isWindowClosing', removeInfo.isWindowClosing);
		// isWindowClosing state can still access window tab info. may work for grabbing just-closed window state
		if (!removeInfo.isWindowClosing) {
			for (var i = 0; i < win.tabs.length; i++) {
				if (win.tabs[i].id != tabId) // don't set icon for this (removed) tab
					chrome.pageAction.setIcon({ tabId:win.tabs[i].id, path:'node-19.png' });
			}
			setDelayedSave(win);
		}
	}
});

chrome.windows.onRemoved.addListener(function(windowId) {
// maybe don't save on close
// win would have to be prevoiusly updated with tabs state since windowId can't be queried at this point
	var win = getTrackedWindow(windowId);
	console.log('windows.onRemoved', win);
	//if (win)
	//	save(win);
	untrackWindow(windowId);
});

/*chrome.browserAction.onClicked.addListener(function(tab) {
	chrome.browserAction.setIcon({'path':'node-icon-fade.png'});
	console.log(tab);
});*/

// stackoverflow.com/questions/15532791/getting-around-x-frame-options-deny-in-a-chrome-extension#15534822
chrome.webRequest.onHeadersReceived.addListener(
	function(info) {
		var headers = info.responseHeaders;
		for (var i=headers.length-1; i>=0; --i) {
			var header = headers[i].name.toLowerCase();
			if (header == 'x-frame-options' || header == 'frame-options') {
				headers.splice(i, 1); // Remove header
			}
		}
		return {responseHeaders: headers};
	},
	{
		urls: [ '*://*/*' ], // Pattern to match all http(s) pages
		types: [ 'sub_frame' ]
	},
	['blocking', 'responseHeaders']
);
